import {
  Button,
  Divider,
  Form,
  Input,
  InputNumber,
  Modal,
  Select,
  Space,
  Tag,
  notification,
} from "antd";
import {
  WebSocketContext,
  SentMessagesCommands,
  GetUuid,
} from "../../../utils";
import { useContext } from "react";
import { InfoCircleOutlined } from "@ant-design/icons";
import TextArea from "antd/es/input/TextArea";
import { useTranslation } from "react-i18next";

export default function GroupTypeSelectionModal({
  isOpen,
  setIsOpen,
  categoryGroup,
  currentSelectedModalGroupType,
  setCurrentSelectedModalGroupType,
}) {
  const [notificationApi, notificationContextHolder] =
    notification.useNotification();
  const webSocketContext = useContext(WebSocketContext);
  const { t } = useTranslation();
  const handleCancel = () => setIsOpen(false);

  const isStartTaskPossible = () => {
    if (
      currentSelectedModalGroupType === null ||
      currentSelectedModalGroupType === undefined
    ) {
      return true;
    }

    let possible = false;

    categoryGroup.groups.forEach((group) => {
      if (group.id === currentSelectedModalGroupType) {
        if (group.tasks === null) {
          possible = true;
          return;
        }
      }
    });

    return possible;
  };

  const handleStartTask = () => {
    let canTaskBeStarted = true;

    const groupTaskDescription = document.getElementById(
      "fitem-grouptypedescription"
    ).value;

    if (groupTaskDescription === "") {
      canTaskBeStarted = false;
    }

    const userSpecifiedGlobalInputForm = document.getElementById(
      "groupTypeSelectionUserSpecifiedGlobalInputForm"
    );

    let globalInputs = [];

    if (canTaskBeStarted && userSpecifiedGlobalInputForm !== null) {
      const userSpecifiedGlobalInputs =
        userSpecifiedGlobalInputForm.getElementsByTagName("input");

      if (userSpecifiedGlobalInputs.length > 0) {
        for (let i = 0; i < userSpecifiedGlobalInputs.length; i++) {
          if (userSpecifiedGlobalInputs[i].value === "") {
            canTaskBeStarted = false;
            break;
          }

          globalInputs.push({
            parameterName: userSpecifiedGlobalInputs[i].id,
            value: userSpecifiedGlobalInputs[i].value,
          });
        }
      }

      const userSpecifiedGlobalTextareas =
        userSpecifiedGlobalInputForm.getElementsByTagName("textarea");

      if (userSpecifiedGlobalTextareas.length > 0) {
        for (let i = 0; i < userSpecifiedGlobalTextareas.length; i++) {
          if (userSpecifiedGlobalTextareas[i].value === "") {
            canTaskBeStarted = false;
            break;
          }

          globalInputs.push({
            parameterName: userSpecifiedGlobalTextareas[i].id,
            value: userSpecifiedGlobalTextareas[i].value,
          });
        }
      }
    }

    if (!canTaskBeStarted) {
      notificationApi["error"]({
        message: t(
          "groupTasks.groupTypeSelectionModal.notification.globalInputsCannotBeEmpty.message"
        ),
        description: t(
          "groupTasks.groupTypeSelectionModal.notification.globalInputsCannotBeEmpty.description"
        ),
      });
      return;
    }

    setIsOpen(false);

    let groupName;
    let numberOfSteps;

    categoryGroup.groups.forEach((group) => {
      if (group.id === currentSelectedModalGroupType) {
        groupName = group.name;
        numberOfSteps = group.tasks.length;
        return;
      }
    });

    const rememberId = GetUuid();

    webSocketContext.StartGroupTasksOpenModalRememberIdRef.current = rememberId;

    webSocketContext.SendSocketMessage(SentMessagesCommands.StartGroupTasks, {
      category: categoryGroup.category,
      id: currentSelectedModalGroupType,
      groupName: groupName,
      description: groupTaskDescription,
      numberOfSteps: parseInt(numberOfSteps),
      globalInputs: globalInputs,
      rememberId: rememberId, // used to open the modal when group task is started by backend and send via websocket back
    });
  };

  return (
    <Modal
      title={t("groupTasks.groupTypeSelectionModal.title")}
      open={isOpen}
      onCancel={handleCancel}
      centered
      maskClosable={false}
      footer={[
        <Space key={0}>
          <Button key="back" onClick={handleCancel}>
            {t("buttonCancel")}
          </Button>

          <Button
            key="submit"
            type="primary"
            disabled={isStartTaskPossible()}
            onClick={handleStartTask}
          >
            {t("groupTasks.groupTypeSelectionModal.button.startTask")}
          </Button>
        </Space>,
      ]}
    >
      {notificationContextHolder}
      <Select
        placeholder={t("groupTasks.groupTypeSelectionModal.select.placeholder")}
        style={{ width: "100%" }}
        value={currentSelectedModalGroupType}
        onChange={(value) => setCurrentSelectedModalGroupType(value)}
      >
        {categoryGroup.groups !== undefined
          ? categoryGroup.groups.map((group, index) => {
              return (
                <Select.Option key={index} value={group.id}>
                  {group.name}
                </Select.Option>
              );
            })
          : null}
      </Select>

      {currentSelectedModalGroupType !== null && (
        <Form layout="vertical">
          <br />
          <Form.Item
            label={t(
              "groupTasks.groupTypeSelectionModal.form.item.groupTypeDescription.label"
            )}
            tooltip={{
              title: t(
                "groupTasks.groupTypeSelectionModal.form.item.groupTypeDescription.tooltip.title"
              ),
              icon: <InfoCircleOutlined />,
            }}
            required
          >
            <Input id="fitem-grouptypedescription" />
          </Form.Item>
        </Form>
      )}

      <GroupGlobalInputs
        categoryGroup={categoryGroup}
        currentSelectedModalGroupType={currentSelectedModalGroupType}
        notificationApi={notificationApi}
      />
    </Modal>
  );
}

function GroupGlobalInputs({
  categoryGroup,
  currentSelectedModalGroupType,
  notificationApi,
}) {
  const { t } = useTranslation();

  const getLabel = (displayName) => {
    return (
      <>
        {displayName}
        <Tag style={{ marginLeft: 6 }} color="purple">
          {t("groupTasks.tag.global")}
        </Tag>
      </>
    );
  };

  var elements = [];

  if (currentSelectedModalGroupType !== undefined) {
    categoryGroup.groups.forEach((group) => {
      if (
        group.id === currentSelectedModalGroupType &&
        group.globalInputs?.length > 0
      ) {
        group.globalInputs.forEach((globalInput) => {
          switch (globalInput.type) {
            case "text":
              elements.push(
                <Form.Item
                  key={"fitem-" + globalInput.parameterName}
                  label={getLabel(globalInput.displayName)}
                  required
                >
                  <Input id={globalInput.parameterName} />
                </Form.Item>
              );
              break;
            case "number":
              elements.push(
                <Form.Item
                  key={"fitem-" + globalInput.parameterName}
                  label={getLabel(globalInput.displayName)}
                  required
                >
                  <InputNumber
                    style={{ width: "100%" }}
                    max={Number.MAX_SAFE_INTEGER}
                    id={globalInput.parameterName}
                  />
                </Form.Item>
              );
              break;
            case "textarea":
              elements.push(
                <Form.Item
                  key={"fitem-" + globalInput.parameterName}
                  label={getLabel(globalInput.displayName)}
                  required
                >
                  <TextArea id={globalInput.parameterName} />
                </Form.Item>
              );
              break;
            default:
              notificationApi["error"]({
                message: t(
                  "groupTasks.groupTypeSelectionModal.notification.globalInputTypeNotImplemented.message",
                  { globalInputType: globalInput.type }
                ),
                description: t(
                  "groupTasks.groupTypeSelectionModal.notification.globalInputTypeNotImplemented.description",
                  { globalInputDisplayName: globalInput.displayName }
                ),
              });
              break;
          }
        });
        return;
      }
    });
  }

  return (
    <>
      {elements.length > 0 ? (
        <div>
          <Divider />
          <h3 style={{ fontWeight: "bold" }}>
            {t("groupTasks.groupTypeSelectionModal.h3")}
          </h3>
          <Form
            layout="vertical"
            id="groupTypeSelectionUserSpecifiedGlobalInputForm"
          >
            {elements}
          </Form>
        </div>
      ) : null}
    </>
  );
}
