import { Button, Popconfirm, Result } from "antd";
import { useContext, useState } from "react";
import GroupTasksViewModal from "./GroupTasksViewModal";
import GroupTypeSelectionModal from "./GroupTypeSelectionModal";
import GroupTaskTableList from "./GroupTasksTableList";
import {
  Constants,
  SentMessagesCommands,
  WebSocketContext,
  hasPermission,
  hasXYPermission,
} from "../../../utils";
import { useTranslation } from "react-i18next";
import { ReloadOutlined } from "@ant-design/icons";

export default function GroupTasks({ isGroupTasksViewModalOpen }) {
  const [isGroupTypeSelectionModalOpen, setIsGroupTypeSelectionModalOpen] =
    useState(false);
  const [currentCategoryGroup, setCurrentCategoryGroup] = useState([]);
  const [currentSelectedModalGroupType, setCurrentSelectedModalGroupType] =
    useState();
  const webSocketContext = useContext(WebSocketContext);
  const { t } = useTranslation();

  const showGroupTypeSelectionModal = (categoryGroup) => {
    setCurrentCategoryGroup(categoryGroup);
    setIsGroupTypeSelectionModalOpen(true);
    setCurrentSelectedModalGroupType(null);
  };

  const filteredCategoryGroups = webSocketContext.CategoryGroups.filter(
    (categoryGroup) =>
      hasXYPermission(
        webSocketContext.User.Permissions,
        Constants.PERMISSIONS.GROUP_TASKS.OVERVIEW.XYView,
        categoryGroup.category
      )
  );

  const onCheckingForCategoryGroupChanges = () => {
    webSocketContext.SendSocketMessage(
      SentMessagesCommands.GroupTasksCheckingForCategoryGroupChanges,
      {}
    );
  };

  return (
    <>
      {webSocketContext.CategoryGroups.length === 0 ? (
        <Result status="404" title={t("groupTasks.categoryGroups.length0")} />
      ) : (
        <>
          {filteredCategoryGroups.length > 0 ? (
            <>
              {filteredCategoryGroups.map((categoryGroup) => {
                return (
                  <GroupTaskTableList
                    key={categoryGroup.category}
                    categoryGroup={categoryGroup}
                    showGroupTypeSelectionModal={showGroupTypeSelectionModal}
                    groupTasks={webSocketContext.GroupTasks}
                  />
                );
              })}
            </>
          ) : (
            <Result
              key="result"
              status="403"
              title={t("groupTasks.categoryGroups.assignedToNoTask.title")}
              subTitle={t("contactAdmin")}
            />
          )}
        </>
      )}

      <GroupTypeSelectionModal
        isOpen={isGroupTypeSelectionModalOpen}
        setIsOpen={setIsGroupTypeSelectionModalOpen}
        categoryGroup={currentCategoryGroup}
        currentSelectedModalGroupType={currentSelectedModalGroupType}
        setCurrentSelectedModalGroupType={setCurrentSelectedModalGroupType}
      />

      <GroupTasksViewModal isOpen={isGroupTasksViewModalOpen} />

      {hasPermission(
        webSocketContext.User.Permissions,
        Constants.PERMISSIONS.GROUP_TASKS.CHECKING_FOR_CATEGORY_GROUP_CHANGES
      ) && (
        <div
          style={{
            display: "flex",
            justifyContent: "center",
            marginTop: 20,
          }}
        >
          <Popconfirm
            placement="top"
            okText={t(
              "groupTasks.button.checkingForCategoryGroupChanges.popover.buttonOk"
            )}
            cancelText={t("buttonCancel")}
            title={t(
              "groupTasks.button.checkingForCategoryGroupChanges.checkingForCategoryGroupChanges.popover.title"
            )}
            onConfirm={() => onCheckingForCategoryGroupChanges()}
          >
            <Button shape="round" icon={<ReloadOutlined />} size="large">
              {t("groupTasks.button.checkingForCategoryGroupChanges")}
            </Button>
          </Popconfirm>
        </div>
      )}
    </>
  );
}
