import { Popconfirm, Space, Table } from "antd";
import {
  FormatDatetime,
  MyAvatar,
  SentMessagesCommands,
  WebSocketContext,
  getUserId,
} from "../../utils";
import { useContext } from "react";
import { Link } from "react-router-dom";
import { Constants } from "../../utils";
import { useTranslation } from "react-i18next";

export default function Scanners() {
  const webSocketContext = useContext(WebSocketContext);
  const { t } = useTranslation();

  const getTableColumns = () => {
    return [
      {
        title: t("scanners.column.name"),
        dataIndex: "name",
        key: "name",
      },
      {
        title: t("scanners.column.usedBy"),
        dataIndex: "usedByUserId",
        key: "usedByUserId",
      },
      {
        title: t("scanners.column.lastUsed"),
        dataIndex: "lastUsed",
        key: "lastUsed",
      },
      {
        title: t("scanners.column.registeredAt"),
        dataIndex: "registeredAt",
        key: "registeredAt",
      },
      {
        title: t("scanners.column.userAgent"),
        dataIndex: "userAgent",
        key: "userAgent",
      },
      {
        title: t("scanners.column.action"),
        dataIndex: "action",
        key: "action",
        render: (_, record) => (
          <Space size="middle">
            {record._usedByUserId === getUserId() ? (
              <Popconfirm
                placement="left"
                title={t("scanners.column.action.disconnect.popconfirm.title")}
                okText={t(
                  "scanners.column.action.disconnect.popconfirm.okText"
                )}
                cancelText={t("buttonCancel")}
                onConfirm={() => {
                  webSocketContext.SendSocketMessage(
                    SentMessagesCommands.ScannersDisconnectScanner,
                    {
                      ScannerId: record.key,
                    }
                  );
                }}
              >
                <Link to="#">{t("scanners.column.action.disconnect")}</Link>
              </Popconfirm>
            ) : (
              <Popconfirm
                placement="left"
                title={t("scanners.column.action.use.popconfirm.title")}
                description={t(
                  "scanners.column.action.use.popconfirm.description"
                )}
                okText={t("scanners.column.action.use.popconfirm.okText")}
                cancelText={t("buttonCancel")}
                onConfirm={() => {
                  webSocketContext.SendSocketMessage(
                    SentMessagesCommands.ScannersUseScanners,
                    {
                      ScannerId: record.key,
                    }
                  );
                }}
              >
                <Link to="#">{t("scanners.column.action.use")}</Link>
              </Popconfirm>
            )}
          </Space>
        ),
      },
    ];
  };

  const getTableItems = () => {
    if (webSocketContext.Scanners === null) return [];

    let items = [];

    webSocketContext.Scanners.forEach((scanner) => {
      const usedByUser = webSocketContext.AllUsers.find(
        (user) => user.Id === scanner.UsedByUserId
      );

      items.push({
        key: scanner.Id,
        name: scanner.Name,
        usedByUserId:
          scanner.UsedByUserId === "" ? (
            Constants.TEXT_EMPTY_PLACEHOLDER
          ) : (
            <>
              <MyAvatar avatar={usedByUser.Avatar} />
              <span> {usedByUser.Username}</span>
            </>
          ),
        _usedByUserId: scanner.UsedByUserId,
        lastUsed: FormatDatetime(scanner.LastUsed),
        registeredAt: FormatDatetime(scanner.RegisteredAt),
        userAgent: scanner.UserAgent,
      });
    });

    return items;
  };

  return (
    <>
      <h1 style={{ fontWeight: "bold", float: "left" }}>
        {t("scanners.header.scanners")} (
        {webSocketContext.Scanners === null
          ? "0"
          : webSocketContext.Scanners.length}
        )
      </h1>

      <Table columns={getTableColumns()} dataSource={getTableItems()} />
    </>
  );
}
